//
//  CallLogController.m
//  CallChecker
//
//  Created by Steffen Kamp on Sat Feb 08 2003.
//  <mailto:steffen@earthlingsoft.net>
//  Copyright (c) 2003 earthlingsoft. All rights reserved.
//

/*
 This file is part of CallChecker.

 CallChecker is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 CallChecker is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with CallChecker; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  US
 */

#import "CallLogController.h"
#import "CallMonitor.h"
#import "Call.h"
#import "Preferences.h"
#import <sys/socket.h>
#import <unistd.h>
#import "SocketConnect.h"

#define READBUFFER 65536

static CallLogController *_sharedInstance = nil;

@implementation CallLogController

+ (CallLogController *)sharedController {
    if(!_sharedInstance) {
        _sharedInstance = [[self alloc] init];
    }
    return _sharedInstance;
}

- (id)init {
    self = [super init];
    if(self) {
        [NSBundle loadNibNamed:@"Call Log" owner:self];
    }
    return self;
}

- (void)dealloc {
    [loggedCalls release];
    [window release];
    [super dealloc];
}

- (void)awakeFromNib {
    NSString *dateFormat = [NSString stringWithFormat:@"%@  %@", [[NSUserDefaults standardUserDefaults] stringForKey:NSShortDateFormatString], [[NSUserDefaults standardUserDefaults] stringForKey:NSTimeFormatString]];
    [[dateColumn dataCell] setFormatter:[[[NSDateFormatter alloc] initWithDateFormat:dateFormat allowNaturalLanguage:NO] autorelease]];
}

- (IBAction)showWindow:(id)sender {
    [self setLoggedCalls:[self getLoggedCalls]];
    [callLogTable reloadData];
    [window makeKeyAndOrderFront:self];
}

- (void)reloadData {
    if([window isVisible])
        [callLogTable reloadData];
}

#pragma mark ----- Call Log Stuff -----

- (NSString *)loggedCallsFromAddress:(NSString *)addr port:(int)port {
    char buffer[READBUFFER];
    int count;
    int sockfd;
    NSMutableString *result = [[[NSMutableString alloc] init] autorelease];

    sockfd = connectedSocketForHostAndPort(addr, port);

    if(sockfd < 0) {
        //NSLog(@"no valid socket");
        return nil;
    }

    if(send(sockfd, "telmond-log-file\r\n", 18, 0) == -1) {
        NSLog(@"'send(telmond-log-file)' failed: %i", errno);
        close(sockfd);
        return nil;
    }

    while( ![result hasSuffix:@"OK 0\r\n"]
           && ![result hasSuffix:@"ERR\r\n"]
           && (count = recv(sockfd, buffer, READBUFFER, 0)) > 0) {
        [result appendString:[NSString stringWithCString:buffer length:count]];
    }

    if(count == -1) {
        NSLog(@"'recv()' failed: %i", errno);
        close(sockfd);
        return nil;
    }

    if(send(sockfd, "quit\r\n", 6, 0) == -1) {
        NSLog(@"'send(quit)' failed: %i", errno);
        close(sockfd);
        return nil;
    }

    while((count = recv(sockfd, buffer, READBUFFER, 0)) > 0) {
        [result appendString:[NSString stringWithCString:buffer length:count]];
    }

    close(sockfd);

    return result;
}

- (NSArray *)getLoggedCalls {
    int i;
    NSMutableArray *calls;
    NSArray *loglines;
    NSString *logfile = [self loggedCallsFromAddress:[DEFAULTS stringForKey:DefaultHost]
                                                port:[DEFAULTS integerForKey:DefaultImondPort]];

    loglines = [logfile componentsSeparatedByString:@"\n"];

    if([loglines count] <= 2)
        return nil;

    calls = [NSMutableArray arrayWithCapacity:[loglines count]];
    for(i=[loglines count]-3; i>=0; --i) {
        Call *call = [Call callFromString:[loglines objectAtIndex:i]];
        if([Preferences reportForMSN:[call msn]])
            [calls addObject:call];
    }
    return calls;
}

- (void)setLoggedCalls:(NSArray *)array {
    [loggedCalls autorelease];
    loggedCalls = [array retain];
}

#pragma mark ----- Call Log DataSource -----

- (int)numberOfRowsInTableView:(NSTableView *)tableView {
    return [loggedCalls count];
}

- (id)tableView:(NSTableView *)tableView objectValueForTableColumn:(NSTableColumn *)tableColumn row:(int)row {
    NSString *identifier = [tableColumn identifier];
    Call *call = [loggedCalls objectAtIndex:row];

    if([identifier isEqualToString:@"name"]) {
        NSString *name = [CallMonitor nameForCallerID:[call callerID]];
        if(name)
            return [NSString stringWithFormat:@"%@ (%@)", name, [call callerID]];
        else
            return [NSString stringWithFormat:@"%@", [call callerID]];
    } else if([identifier isEqualToString:@"msn"]) {
        return [Preferences aliasForMSN:[call msn]];
    } else if([identifier isEqualToString:@"date"]) {
        return [call date];
    }
    return @"?";
}

@end
