//
//  CallMonitor.m
//  CallChecker
//
//  Created by Steffen Kamp on Mon Oct 19 2002.
//  <mailto:steffen@earthlingsoft.net>
//  Copyright (c) 2002 earthlingsoft. All rights reserved.
//

/*
 This file is part of CallChecker.

 CallChecker is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 CallChecker is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with CallChecker; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  US
 */

#import "CallMonitor.h"
#import "Call.h"
#import "CallController.h"
#import <AddressBook/AddressBook.h>
#import <sys/socket.h>
#import <unistd.h>
#import "Preferences.h"
#import "CallLogController.h"
#import "SocketConnect.h"

#define READBUFFER 1024


static NSDictionary *_phonenumberDict = nil;

@implementation CallMonitor

- (id)init {
    self = [super init];
    if(self) {
        lastMessage = nil;
        failIconSet = NO;
        [self updatePhonenumbers:nil];
        [[NSNotificationCenter defaultCenter] addObserver:self
                                                 selector:@selector(updatePhonenumbers:)
                                                     name:kABDatabaseChangedExternallyNotification
                                                   object:nil];
    }
    return self;
}

- (void)dealloc {
    [lastCallTimer release];
    [lastMessage release];
    [super dealloc];
}

- (void)awakeFromNib {
    NSString *hostString = [DEFAULTS stringForKey:DefaultHost];
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(hostDefaultsChanged:)
                                                 name:CCHostDefaultsChangedNotification
                                               object:nil];
    if(!hostString || [hostString isEqualToString:@""])
        [[Preferences sharedPreferences] showWindow:self];
    else
        [self scheduleLastCallTimer];
}

- (void)applicationWillTerminate:(NSNotification *)notification {
    [NSApp setApplicationIconImage:[NSImage imageNamed:@"CallChecker"]];
}

- (void)hostDefaultsChanged:(NSNotification *)notification {
    // invalidate the timer
    [self setLastCallTimer:nil];

    // check if host defaults are ok
    if(![self getLastCall]) {
        NSRunAlertPanel(NSLocalizedString(@"Could not connect to router", @"Error Title"),
                        NSLocalizedString(@"Please verify the IP address and port numbers and make sure the router is running.", @"Error Message"),
                        @"", @"", @"");
    } else {
        [self scheduleLastCallTimer];
    }
}


#pragma mark ----- Last Call Stuff -----

- (void)setLastCallTimer:(NSTimer *)timer {
    [lastCallTimer invalidate];
    [lastCallTimer autorelease];
    lastCallTimer = [timer retain];
}

- (void)scheduleLastCallTimer {
    NSTimer *timer = [NSTimer scheduledTimerWithTimeInterval:[DEFAULTS floatForKey:DefaultPollingInterval] target:self selector:@selector(getLastCallWithTimer:) userInfo:nil repeats:NO];
    [self setLastCallTimer:timer];
}

- (void)getLastCallWithTimer:(NSTimer *)timer {
    [self getLastCall];
    [self scheduleLastCallTimer];
}

- (BOOL)getLastCall {
    if(![self getLastCallFromAddress:[DEFAULTS stringForKey:DefaultHost]
                                port:[DEFAULTS integerForKey:DefaultPort]]) {
        if(!failIconSet) {
            failIconSet = YES;
            [NSApp setApplicationIconImage:[NSImage imageNamed:@"CallChecker Fail"]];
        }
        return NO;
    } else {
        if(failIconSet) {
            failIconSet = NO;
            [NSApp setApplicationIconImage:[NSImage imageNamed:@"CallChecker"]];
        }
        return YES;
    }
}


- (BOOL)getLastCallFromAddress:(NSString *)addr port:(int)port {
    char buffer[READBUFFER];
    int count;
    int sockfd;
    NSMutableString *result = [[[NSMutableString alloc] init] autorelease];

    sockfd = connectedSocketForHostAndPort(addr, port);

    if(sockfd < 0) {
        //NSLog(@"no valid socket");
        return NO;
    }
    
    while((count = recv(sockfd, buffer, READBUFFER, 0)) > 0) {
        [result appendString:[NSString stringWithCString:buffer length:count]];
    }

    if(count == -1) {
        NSLog(@"'recv()' failed: %i", errno);
        close(sockfd);
        return NO;
    }

    close(sockfd);

    [self handleCall:result];
    return YES;
}

- (void)handleCall:(NSString *)string {
    if(lastMessage == nil || ![lastMessage isEqualToString:string]) {
        Call *call = [Call callFromString:string];

        if(call) {
            if([Preferences reportForMSN:[call msn]]) {
                [[CallController alloc] initWithCall:call];
            }
    
            [lastMessage autorelease];
            lastMessage = [string copy];
        }
    }
}


#pragma mark ----- Address Book Stuff -----

- (void)updatePhonenumbers:(NSNotification *)notification {
    NSArray *persons = [[ABAddressBook sharedAddressBook] people];
    ABPerson *person;
    NSEnumerator *enm = [persons objectEnumerator];
    NSMutableDictionary *numberDict = [NSMutableDictionary dictionaryWithCapacity:[persons count]];
    
    while (person = [enm nextObject]) {
        ABMultiValue *numbers = [person valueForProperty:kABPhoneProperty];
        int i;
        for(i = 0; i < [numbers count]; ++i) {
            NSString *number = [numbers valueAtIndex:i];
            [numberDict setObject:[person uniqueId] forKey:[self trimPhonenumber:number]];
        }
    }

    [self setPhonenumberDictionary:numberDict];

    [[CallLogController sharedController] reloadData];
}

- (void)setPhonenumberDictionary:(NSDictionary *)dict {
    [_phonenumberDict autorelease];
    _phonenumberDict = [dict retain];
}

+ (NSDictionary *)phonenumberDict {
    return _phonenumberDict;
}

+ (NSString *)nameForCallerID:(NSString *)number {
    ABPerson *person;
    NSString *firstName;
    NSString *lastName;
    
    if([number isEqualToString:@"00"])
        return nil;
    
    person = [self personForPhonenumber:number];
    if(!person)
        return nil;

    firstName = [person valueForProperty:kABFirstNameProperty];
    lastName = [person valueForProperty:kABLastNameProperty];
    if(firstName != nil && lastName != nil)
        return [NSString stringWithFormat:@"%@ %@", firstName, lastName];
    else if(firstName != nil)
        return firstName;
    else if(lastName != nil)
        return lastName;
    else
        return nil;
}

+ (ABPerson *)personForPhonenumber:(NSString *)number {
    NSEnumerator *enm = [[self phonenumberDict] keyEnumerator];
    NSString *dictNumber;
    
    NSString *trimmedNumber;
    if([number hasPrefix:@"0"])
        trimmedNumber = [number substringFromIndex:1];
    else
        trimmedNumber = number;

    while(dictNumber = [enm nextObject]) {
        if([dictNumber hasSuffix:trimmedNumber])
            return (ABPerson *)[[ABAddressBook sharedAddressBook] recordForUniqueId:[[self phonenumberDict] objectForKey:dictNumber]];
    }

    return nil;
}

- (NSString *)trimPhonenumber:(NSString *)number {
    NSScanner *scnr = [NSScanner scannerWithString:number];
    NSMutableString *outNumber = [[[NSMutableString alloc] init] autorelease];
    NSString *digits;
    NSCharacterSet *set = [[NSCharacterSet decimalDigitCharacterSet] invertedSet];
    [scnr setCharactersToBeSkipped:set];

    while (![scnr isAtEnd]) {
        [scnr scanUpToCharactersFromSet:set intoString:&digits];
        [outNumber appendString:digits];
    }

    return outNumber;
}


#pragma mark ----- Menu Handling -----

- (IBAction)menuShowHelp:(id)sender {
    NSString *helpName = [[[NSBundle mainBundle] localizedInfoDictionary] objectForKey:@"NSHelpFile"];
    NSString *path;

    if(!helpName || [helpName isEqual:@""]) {
        [NSApp showHelp:self];
        return;
    }

    path = [[NSBundle mainBundle] pathForResource:helpName ofType:@""];
    if(!path || [path isEqual:@""]) {
        [NSApp showHelp:self];
        return;
    }

    [[NSWorkspace sharedWorkspace] openFile:path];
}

- (IBAction)menuShowCallLog:(id)sender {
    [[CallLogController sharedController] showWindow:self];
}

- (IBAction)menuShowLastCall:(id)sender {
    Call *call = [Call callFromString:lastMessage];

    if(call) {
        [[CallController alloc] initWithCall:call];
    } else {
        NSBeep();
    }
}

- (IBAction)menuShowPreferences:(id)sender {
    [[Preferences sharedPreferences] showWindow:self];
}

@end
