//
//  Preferences.m
//  CallChecker
//
//  Created by Steffen Kamp on Mon Oct 19 2002.
//  <mailto:steffen@earthlingsoft.net>
//  Copyright (c) 2002 earthlingsoft. All rights reserved.
//

/*
 This file is part of CallChecker.

 CallChecker is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 CallChecker is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with CallChecker; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  US
 */

#import "Preferences.h"


NSString *DefaultHost = @"Host";
NSString *DefaultPort = @"Port";
NSString *DefaultReportAll = @"Report all MSN";
NSString *DefaultMSNAliases = @"MSN Aliases";
NSString *DefaultPollingInterval = @"Polling Interval";
NSString *DefaultImondPort = @"imond Port";

NSString *CCAliasMSNKey = @"MSN";
NSString *CCAliasNameKey = @"Alias";
NSString *CCAliasActiveKey = @"active";

NSString *CCHostDefaultsChangedNotification = @"HostDefaultsChanged";

// No Drag&Drop yet
// NSString *CCAliasPboardType = @"CCAliasPboardType";

static Preferences *_sharedInstance = nil;

@implementation Preferences

+ (Preferences *)sharedPreferences {
    if(!_sharedInstance) {
        _sharedInstance = [[self alloc] init];
    }
    return _sharedInstance;
}

- (id)init {
    self = [super init];
    if(self) {
        [NSBundle loadNibNamed:@"Preferences" owner:self];
    }
    return self;
}

- (void)dealloc {
    [window release];
    [super dealloc];
}

- (void)awakeFromNib {
    NSTableColumn *activeColumn = [aliasTable tableColumnWithIdentifier:@"active"];
    NSButtonCell *checkBox = [[[NSButtonCell alloc] init] autorelease];
    [checkBox setButtonType:NSSwitchButton];
    [checkBox setTitle:@""];
    [checkBox setControlSize:NSSmallControlSize];
    [activeColumn setDataCell:checkBox];

    // No Drag & Drop yet
    //[aliasTable registerForDraggedTypes:[NSArray arrayWithObject:CCAliasPboardType]];
    
    [self registerDefaults];
    [self fillInDefaults];
    [activateSettings setEnabled:NO];
}

- (void)registerDefaults {
    [DEFAULTS registerDefaults:[NSDictionary dictionaryWithObjectsAndKeys:
        [NSNumber numberWithInt:1], DefaultReportAll,
        [NSNumber numberWithInt:5001], DefaultPort,
        [NSNumber numberWithInt:5000], DefaultImondPort,
        [NSNumber numberWithDouble:2.], DefaultPollingInterval,
        @"", DefaultHost,
        nil, nil]];
}

- (void)updateHostDefaults {
    // MSN Aliases are handled by the table dataSource methods
    [DEFAULTS setObject:[host stringValue] forKey:DefaultHost];
    [DEFAULTS setInteger:[port intValue] forKey:DefaultPort];
    [DEFAULTS setInteger:[imondPort intValue] forKey:DefaultImondPort];
    [DEFAULTS setFloat:[pollingInterval floatValue] forKey:DefaultPollingInterval];

    [[NSNotificationCenter defaultCenter] postNotificationName:CCHostDefaultsChangedNotification object:self];
    [activateSettings setEnabled:NO];
}

- (void)fillInDefaults {
    [imondPort setIntValue:[DEFAULTS integerForKey:DefaultImondPort]];
    [host setStringValue:[DEFAULTS stringForKey:DefaultHost]];
    [port setIntValue:[DEFAULTS integerForKey:DefaultPort]];
    [pollingInterval setFloatValue:[DEFAULTS floatForKey:DefaultPollingInterval]];
    if([DEFAULTS boolForKey:DefaultReportAll])
        [reportAll setState:NSOnState];
    else
        [reportAll setState:NSOffState];
}


#pragma mark ----- Action Handlers -----

- (IBAction)activateSettingsAction:(id)sender {
    [activateSettings setEnabled:NO];
    [self updateHostDefaults];
}

- (IBAction)addAliasAction:(id)sender {
    NSMutableArray *aliasArray = [[[DEFAULTS arrayForKey:DefaultMSNAliases] mutableCopy] autorelease];
    NSDictionary *aliasEntry;

    if(!aliasArray)
        aliasArray = [NSMutableArray arrayWithCapacity:5];

    aliasEntry = [NSDictionary dictionaryWithObjectsAndKeys:@"0", CCAliasMSNKey,
        @"Name", CCAliasNameKey,
        [NSNumber numberWithInt:1], CCAliasActiveKey,
        nil];

    [aliasArray addObject:aliasEntry];

    [DEFAULTS setObject:aliasArray forKey:DefaultMSNAliases];

    [aliasTable reloadData];

    [aliasTable selectRow:([aliasTable numberOfRows]-1) byExtendingSelection:NO];
    [aliasTable editColumn:[aliasTable columnWithIdentifier:@"MSN"] row:([aliasTable numberOfRows]-1) withEvent:nil select:YES];
}

- (IBAction)removeAliasAction:(id)sender {
    NSMutableArray *aliasArray = [[[DEFAULTS arrayForKey:DefaultMSNAliases] mutableCopy] autorelease];
    int row;
    
    while((row = [aliasTable selectedRow]) >= 0) {
        [aliasTable deselectRow:row];
        [aliasArray removeObjectAtIndex:row];
    }
    
    [DEFAULTS setObject:aliasArray forKey:DefaultMSNAliases];

    [aliasTable reloadData];
}

- (IBAction)reportAllAction:(id)sender {
    [DEFAULTS setBool:([reportAll state]==NSOnState) forKey:DefaultReportAll];
}

- (IBAction)showWindow:(id)sender {
    if(window == nil) {
        [NSBundle loadNibNamed:@"Preferences" owner:self];
        [self fillInDefaults];
    }
    [self tableViewSelectionDidChange:nil];
    [window makeKeyAndOrderFront:self];
}


#pragma mark ----- MSN Aliases Data Source -----

- (int)numberOfRowsInTableView:(NSTableView *)tableView {
    return [[DEFAULTS arrayForKey:DefaultMSNAliases] count];
}
- (id)tableView:(NSTableView *)tableView objectValueForTableColumn:(NSTableColumn *)tableColumn row:(int)row {
    NSDictionary *aliasEntry = [[DEFAULTS arrayForKey:DefaultMSNAliases] objectAtIndex:row];
    NSString *identifier = [tableColumn identifier];
    return [aliasEntry objectForKey:identifier];
}
- (void)tableView:(NSTableView *)tableView setObjectValue:(id)object forTableColumn:(NSTableColumn *)tableColumn row:(int)row {
    NSMutableArray *aliasArray = [[[DEFAULTS arrayForKey:DefaultMSNAliases] mutableCopy] autorelease];
    NSDictionary *aliasEntry;
    NSString *identifier = [tableColumn identifier];
    NSString *alias, *msn;
    NSNumber *active;

    if([identifier isEqualToString:@"MSN"]) {
        msn = object;
        alias = [[tableView dataSource] tableView:tableView objectValueForTableColumn:[tableView tableColumnWithIdentifier:@"Alias"] row:row];
        active = [[tableView dataSource] tableView:tableView objectValueForTableColumn:[tableView tableColumnWithIdentifier:@"active"] row:row];
    } else if([identifier isEqualToString:@"Alias"]) {
        alias = object;
        msn = [[tableView dataSource] tableView:tableView objectValueForTableColumn:[tableView tableColumnWithIdentifier:@"MSN"] row:row];
        active = [[tableView dataSource] tableView:tableView objectValueForTableColumn:[tableView tableColumnWithIdentifier:@"active"] row:row];
    } else if([identifier isEqualToString:@"active"]) {
        active = object;
        alias = [[tableView dataSource] tableView:tableView objectValueForTableColumn:[tableView tableColumnWithIdentifier:@"Alias"] row:row];
        msn = [[tableView dataSource] tableView:tableView objectValueForTableColumn:[tableView tableColumnWithIdentifier:@"MSN"] row:row];
    } else {
        NSLog(@"Illegal column");
        return;
    }

    aliasEntry = [NSDictionary dictionaryWithObjectsAndKeys:msn, @"MSN",
        alias, @"Alias",
        active, @"active", nil];

    [aliasArray replaceObjectAtIndex:row withObject:aliasEntry];

    [DEFAULTS setObject:aliasArray forKey:DefaultMSNAliases];
}

/*
 // Drag&Drop not working yet
 // actually, soring might be preferable to dragging

- (BOOL)tableView:(NSTableView *)tv writeRows:(NSArray*)rows toPasteboard:(NSPasteboard*)pboard {
    // This method is called after it has been determined that a drag should begin, but before the drag has been started.  To refuse the drag, return NO.  To start a drag, return YES and place the drag data onto the pasteboard (data, owner, etc...).  The drag image and other drag related information will be set up and provided by the table view once this call returns with YES.  The rows array is the list of row numbers that will be participating in the drag.
    NSArray *aliasArray = [DEFAULTS arrayForKey:DefaultMSNAliases];
    NSMutableArray *dragData = [NSMutableArray arrayWithCapacity:[rows count]];
    int i;
    for(i=0; i<[rows count]; ++i) {
        [dragData addObject:[aliasArray objectAtIndex:i]];
    }

    [pboard declareTypes:[NSArray arrayWithObject:CCAliasPboardType] owner:self];

    if(![pboard setPropertyList:dragData forType:CCAliasPboardType]) {
        NSLog(@"drag error");
        return NO;
    }

    return YES;
}

- (NSDragOperation)tableView:(NSTableView*)tv validateDrop:(id <NSDraggingInfo>)info proposedRow:(int)row proposedDropOperation:(NSTableViewDropOperation)op {
// This method is used by NSTableView to determine a valid drop target.  Based on the mouse position, the table view will suggest a proposed drop location.  This method must return a value that indicates which dragging operation the data source will perform.  The data source may "re-target" a drop if desired by calling setDropRow:dropOperation: and returning something other than NSDragOperationNone.  One may choose to re-target for various reasons (eg. for better visual feedback when inserting into a sorted position).
    if(op == NSTableViewDropOn)
        return NSDragOperationNone;
    return NSDragOperationMove;
}

- (BOOL)tableView:(NSTableView*)tv acceptDrop:(id <NSDraggingInfo>)info row:(int)row dropOperation:(NSTableViewDropOperation)op {
    // This method is called when the mouse is released over an outline view that previously decided to allow a drop via the validateDrop method.  The data source should incorporate the data from the dragging pasteboard at this time.
    return NO;
}
*/


#pragma mark ----- MSN Aliases Table Delegate -----

- (void)tableViewSelectionDidChange:(NSNotification *)notification {
    if([aliasTable numberOfSelectedRows] > 0)
        [removeAlias setEnabled:YES];
    else
        [removeAlias setEnabled:NO];
}

#pragma mark ----- TextField Delegate -----

- (void)controlTextDidChange:(NSNotification *)notification {
    // -textDidChange: is never invoked as these are all NSTextFields
    id sender = [notification object];
    if(sender == host ||
       sender == port ||
       sender == imondPort ||
       sender == pollingInterval) {
        [activateSettings setEnabled:YES];
    }
}


#pragma mark ----- Window delegate -----

- (BOOL)windowShouldClose:(NSWindow *)aWindow {
    if(aWindow != window)
        return YES;
    if([activateSettings isEnabled]) {
        NSBeginAlertSheet(NSLocalizedString(@"Activate Network Settings?", @"Dialog Title"),
                          NSLocalizedString(@"Activate", @"Dialog Button"),
                          NSLocalizedString(@"Don't Activate", @"Dialog Button"),
                          NSLocalizedString(@"Cancel", @"Dialog Button"),
                          window,
                          self,
                          @selector(activateSettingsSheetDidEnd:returnCode:contextInfo:),
                          nil,
                          nil,
                          NSLocalizedString(@"You have modified the network settings. Do you want to activate the new settings?", @"Dialog Message"));
        return NO;
    }
    
    return YES;
}

- (void)activateSettingsSheetDidEnd:(NSWindow *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo {
    [sheet orderOut:self];
    if(returnCode == NSAlertDefaultReturn) {
        [window close];
    } else if(returnCode == NSAlertAlternateReturn) {
        [self fillInDefaults];
        [window close];
    }
}

- (void)windowWillClose:(NSNotification *)notification {
    [self updateHostDefaults];
    [DEFAULTS synchronize];
}


#pragma mark ----- Reporting Tests -----

+ (BOOL)reportForAllMSNs {
    if([DEFAULTS boolForKey:DefaultReportAll]
       || [DEFAULTS arrayForKey:DefaultMSNAliases] == nil
       || [[DEFAULTS arrayForKey:DefaultMSNAliases] count] == 0) {
        return YES;
    }
    return NO;
}

+ (BOOL)reportForMSN:(NSString *)msn {
    NSEnumerator *enm;
    NSDictionary *aliasEntry;

    if([self reportForAllMSNs])
        return YES;

    enm = [[DEFAULTS arrayForKey:DefaultMSNAliases] objectEnumerator];
    while(aliasEntry = [enm nextObject]) {
        if([[aliasEntry objectForKey:CCAliasMSNKey] isEqualToString:msn]
           && [[aliasEntry objectForKey:CCAliasActiveKey] boolValue] == YES)
            return YES;
    }

    return NO;
}

+ (int)numberOfActiveMSNs {
    NSArray *aliasArray = [DEFAULTS arrayForKey:DefaultMSNAliases];
    NSEnumerator *enm = [aliasArray objectEnumerator];
    int count = 0;
    NSDictionary *dict;

    while(dict = [enm nextObject]) {
        if([[dict objectForKey:CCAliasActiveKey] boolValue])
            ++count;
    }

    return count;
}



#pragma mark ----- MSN Aliases -----

+ (NSString *)aliasForMSN:(NSString *)msn {
    NSDictionary *aliasEntry = [self aliasEntryForMSN:msn];
    if(!aliasEntry)
        return msn;
    return [aliasEntry objectForKey:CCAliasNameKey];
}

+ (BOOL)isActiveMSN:(NSString *)msn {
    NSDictionary *aliasEntry = [self aliasEntryForMSN:msn];
    if(!aliasEntry)
        return NO;
    return [[aliasEntry objectForKey:CCAliasActiveKey] boolValue];
}

+ (NSDictionary *)aliasEntryForMSN:(NSString *)msn {
    NSEnumerator *enm = [[DEFAULTS arrayForKey:DefaultMSNAliases] objectEnumerator];
    NSDictionary *aliasEntry;

    while(aliasEntry = [enm nextObject]) {
        if([[aliasEntry objectForKey:CCAliasMSNKey] isEqualToString:msn])
            return aliasEntry;
    }

    return nil;
}

@end
